@@ -37,9 +37,9 @@
 #include <chrono>
 #include <set>
 
-#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+#ifdef OPENDHT_JSONCPP
 #include <json/json.h>
-#endif //OPENDHT_PROXY_SERVER
+#endif
 
 namespace dht {
 
@@ -109,6 +109,19 @@ struct OPENDHT_PUBLIC ValueType {
     EditPolicy editPolicy {DEFAULT_EDIT_POLICY};
 };
 
+class TypeStore {
+public:
+    void registerType(const ValueType& type) {
+        types[type.id] = type;
+    }
+    const ValueType& getType(ValueType::Id type_id) const {
+        const auto& t_it = types.find(type_id);
+        return (t_it == types.end()) ? ValueType::USER_DATA : t_it->second;
+    }
+private:
+    std::map<ValueType::Id, ValueType> types {};
+};
+
 /**
  * A "value" is data potentially stored on the Dht, with some metadata.
  *
@@ -174,6 +187,15 @@ struct OPENDHT_PUBLIC Value
                 return f1(v) or f2(v);
             };
         }
+        std::vector<Sp<Value>> filter(const std::vector<Sp<Value>>& values) {
+            if (not (*this))
+                return values;
+            std::vector<Sp<Value>> ret;
+            for (const auto& v : values)
+                if ((*this)(v))
+                    ret.emplace_back(v);
+            return ret;
+        }
     };
 
     /* Sneaky functions disguised in classes */
@@ -350,13 +372,13 @@ struct OPENDHT_PUBLIC Value
     Value(ValueType::Id t, const uint8_t* dat_ptr, size_t dat_len, Id id = INVALID_ID)
      : id(id), type(t), data(dat_ptr, dat_ptr+dat_len) {}
 
-#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+#ifdef OPENDHT_JSONCPP
     /**
      * Build a value from a json object
      * @param json
      */
     Value(Json::Value& json);
-#endif //OPENDHT_PROXY_SERVER
+#endif
 
     template <typename Type>
     Value(ValueType::Id t, const Type& d, Id id = INVALID_ID)
@@ -429,7 +451,7 @@ struct OPENDHT_PUBLIC Value
         return ss.str();
     }
 
-#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+#ifdef OPENDHT_JSONCPP
     /**
      * Build a json object from a value
      * Example:
@@ -439,7 +461,7 @@ struct OPENDHT_PUBLIC Value
      * }
      */
     Json::Value toJson() const;
-#endif //OPENDHT_PROXY_SERVER
+#endif
 
     /** Return the size in bytes used by this value in memory (minimum). */
     size_t size() const;
@@ -820,6 +842,10 @@ struct OPENDHT_PUBLIC Where
         return ss.str();
     }
 
+    bool empty() const {
+        return filters_.empty();
+    }
+
     OPENDHT_PUBLIC friend std::ostream& operator<<(std::ostream& s, const dht::Where& q);
 
 private:
@@ -970,6 +996,10 @@ unpackVector(const std::vector<std::shared_ptr<Value>>& vals) {
     return ret;
 }
 
+#ifdef OPENDHT_JSONCPP
+uint64_t unpackId(const Json::Value& json, const std::string& key);
+#endif
+
 }
 
 MSGPACK_ADD_ENUM(dht::Value::Field)
