@@ -21,6 +21,8 @@
 #include "value.h"
 #include "request.h"
 #include "listener.h"
+#include "value_cache.h"
+#include "op_cache.h"
 
 namespace dht {
 
@@ -59,6 +61,17 @@ struct Dht::SearchNode {
      */
     using SyncStatus = std::map<Sp<Query>, Sp<net::Request>>;
 
+    struct CachedListenStatus {
+        ValueCache cache;
+        Sp<Scheduler::Job> cacheExpirationJob {};
+        Sp<net::Request> req {};
+        CachedListenStatus(ValueStateCallback&& cb) : cache(std::forward<ValueStateCallback>(cb)) {}
+        CachedListenStatus(CachedListenStatus&&) = default;
+        CachedListenStatus(const CachedListenStatus&) = delete;
+        CachedListenStatus& operator=(const CachedListenStatus&) = delete;
+    };
+    using NodeListenerStatus = std::map<Sp<Query>, CachedListenStatus>;
+
     Sp<Node> node {};                 /* the node info */
 
     /* queries sent for finding out values hosted by the node */
@@ -67,7 +80,7 @@ struct Dht::SearchNode {
     std::map<Sp<Query>, std::vector<Sp<Query>>> pagination_queries {};
 
     SyncStatus getStatus {};    /* get/sync status */
-    SyncStatus listenStatus {}; /* listen status */
+    NodeListenerStatus listenStatus {}; /* listen status */
     AnnounceStatus acked {};    /* announcement status for a given value id */
 
     Blob token {};                                 /* last token the node sent to us after a get request */
@@ -123,7 +136,7 @@ struct Dht::SearchNode {
      *
      * @return true if we can send get, else false.
      */
-    bool canGet(time_point now, time_point update, Sp<Query> q = {}) const {
+    bool canGet(time_point now, time_point update, const Sp<Query>& q) const {
         if (node->isExpired())
             return false;
 
@@ -134,9 +147,9 @@ struct Dht::SearchNode {
             if (s.second and s.second->pending())
                 pending = true;
             if (s.first and q and q->isSatisfiedBy(*s.first) and s.second) {
-                if (s.second->pending() and not pending_sq_status)
+                if (s.second->pending())
                     pending_sq_status = true;
-                if (s.second->completed() and not (update > s.second->reply_time) and not completed_sq_status)
+                else if (s.second->completed() and not (update > s.second->reply_time))
                     completed_sq_status = true;
                 if (completed_sq_status and pending_sq_status)
                     break;
@@ -144,7 +157,7 @@ struct Dht::SearchNode {
         }
 
         return (not pending and now > last_get_reply + Node::NODE_EXPIRE_TIME) or
-                not (hasStartedPagination(q) or completed_sq_status or pending_sq_status);
+                not (completed_sq_status or pending_sq_status or hasStartedPagination(q));
     }
 
     /**
@@ -201,6 +214,25 @@ struct Dht::SearchNode {
         getStatus.clear();
     }
 
+    void onValues(const Sp<Query>& q, net::RequestAnswer&& answer, const TypeStore& types, Scheduler& scheduler)
+    {
+        auto l = listenStatus.find(q);
+        if (l != listenStatus.end()) {
+            auto next = l->second.cache.onValues(answer.values,
+                                     answer.refreshed_values,
+                                     answer.expired_values, types, scheduler.time());
+            scheduler.edit(l->second.cacheExpirationJob, next);
+        }
+    }
+
+    void expireValues(const Sp<Query>& q, Scheduler& scheduler) {
+        auto l = listenStatus.find(q);
+        if (l != listenStatus.end()) {
+            auto next = l->second.cache.expireValues(scheduler.time());
+            scheduler.edit(l->second.cacheExpirationJob, next);
+        }
+    }
+
     /**
      * Tells if a request in the status map is expired.
      *
@@ -228,6 +260,12 @@ struct Dht::SearchNode {
                 return r.second and r.second->pending();
             }) != status.end();
     }
+    static bool pending(const NodeListenerStatus& status) {
+        return std::find_if(status.begin(), status.end(),
+            [](const NodeListenerStatus::value_type& r){
+                return r.second.req and r.second.req->pending();
+            }) != status.end();
+    }
 
     bool pendingGet() const { return pending(getStatus); }
 
@@ -263,20 +301,20 @@ struct Dht::SearchNode {
         else
             return isListening(now, ls);
     }
-    bool isListening(time_point now, SyncStatus::const_iterator listen_status) const {
-        if (listen_status == listenStatus.end())
+    bool isListening(time_point now, NodeListenerStatus::const_iterator listen_status) const {
+        if (listen_status == listenStatus.end() or not listen_status->second.req)
             return false;
-        return listen_status->second->reply_time + LISTEN_EXPIRE_TIME > now;
+        return listen_status->second.req->reply_time + LISTEN_EXPIRE_TIME > now;
     }
     void cancelListen() {
         for (const auto& status : listenStatus)
-            node->cancelRequest(status.second);
+            node->cancelRequest(status.second.req);
         listenStatus.clear();
     }
     void cancelListen(const Sp<Query>& query) {
         auto it = listenStatus.find(query);
         if (it != listenStatus.end()) {
-            node->cancelRequest(it->second);
+            node->cancelRequest(it->second.req);
             listenStatus.erase(it);
         }
     }
@@ -293,7 +331,7 @@ struct Dht::SearchNode {
         return ((gs != getStatus.cend() and gs->second and gs->second->pending())
                 or ack == acked.cend() or not ack->second.first or ack->second.first->pending())
                 ? time_point::max()
-                : ack->second.second - REANNOUNCE_MARGIN;
+                : (ack->second.first->completed() ? ack->second.second - REANNOUNCE_MARGIN : time_point::min());
     }
 
     /**
@@ -302,10 +340,10 @@ struct Dht::SearchNode {
      */
     time_point getListenTime(const Sp<Query>& q) const {
         auto listen_status = listenStatus.find(q);
-        if (listen_status == listenStatus.end() or not listen_status->second)
+        if (listen_status == listenStatus.end() or not listen_status->second.req)
             return time_point::min();
-        return listen_status->second->pending() ? time_point::max() :
-            listen_status->second->reply_time + LISTEN_EXPIRE_TIME - REANNOUNCE_MARGIN;
+        return listen_status->second.req->pending() ? time_point::max() :
+            listen_status->second.req->reply_time + LISTEN_EXPIRE_TIME - REANNOUNCE_MARGIN;
     }
 
     /**
@@ -340,10 +378,16 @@ struct Dht::Search {
     std::multimap<time_point, Get> callbacks {};
 
     /* listeners */
-    std::map<size_t, LocalListener> listeners {};
+    std::map<size_t, SearchListener> listeners {};
     size_t listener_token = 1;
 
+    /* Cache */
+    SearchCache cache;
+    Sp<Scheduler::Job> opExpirationJob;
+
     ~Search() {
+        if (opExpirationJob)
+            opExpirationJob->cancel();
         for (auto& get : callbacks) {
             get.second.done_cb(false, {});
             get.second.done_cb = {};
@@ -386,7 +430,8 @@ struct Dht::Search {
      *
      * @param query  The query identifying a 'get' request.
      */
-    time_point getLastGetTime(Sp<Query> query = {}) const;
+    time_point getLastGetTime(const Query&) const;
+    time_point getLastGetTime() const;
 
     /**
      * Is this get operation done ?
@@ -431,19 +476,38 @@ struct Dht::Search {
     bool isAnnounced(Value::Id id) const;
     bool isListening(time_point now) const;
 
-    void cancelListen(size_t token) {
-        Sp<Query> query;
-        const auto& ll = listeners.find(token);
-        if (ll != listeners.cend()) {
-            query = ll->second.query;
-            listeners.erase(ll);
-        }
-        for (auto& sn : nodes) {
-            if (listeners.empty())
-                sn.cancelListen();
-            else if (query)
-                sn.cancelListen(query);
-        }
+    size_t listen(ValueCallback cb, Value::Filter f, const Sp<Query>& q, Scheduler& scheduler) {
+        //DHT_LOG.e(id, "[search %s IPv%c] listen", id.toString().c_str(), (af == AF_INET) ? '4' : '6');
+        return cache.listen(cb, q, f, [&](const Sp<Query>& q, ValueCallback vcb){
+            done = false;
+            auto token = ++listener_token;
+            listeners.emplace(token, SearchListener{q, f, vcb});
+            scheduler.edit(nextSearchStep, scheduler.time());
+            return token;
+        });
+    }
+
+    void cancelListen(size_t token, Scheduler& scheduler) {
+        cache.cancelListen(token, scheduler.time());
+        if (not opExpirationJob)
+            opExpirationJob = scheduler.add(time_point::max(), [this,&scheduler]{
+                auto nextExpire = cache.expire(scheduler.time(), [&](size_t t){
+                    Sp<Query> query;
+                    const auto& ll = listeners.find(t);
+                    if (ll != listeners.cend()) {
+                        query = ll->second.query;
+                        listeners.erase(ll);
+                    }
+                    for (auto& sn : nodes) {
+                        if (listeners.empty())
+                            sn.cancelListen();
+                        else if (query)
+                            sn.cancelListen(query);
+                    }
+                });
+                scheduler.edit(opExpirationJob, nextExpire);
+            });
+        scheduler.edit(opExpirationJob, cache.getExpiration());
     }
 
     /**
@@ -683,11 +747,20 @@ Dht::Search::isSynced(time_point now) const
 }
 
 time_point
-Dht::Search::getLastGetTime(Sp<Query> q) const
+Dht::Search::getLastGetTime(const Query& q) const
 {
     time_point last = time_point::min();
     for (const auto& g : callbacks)
-        last = std::max(last, (not q or q->isSatisfiedBy(*g.second.query) ? g.second.start : time_point::min()));
+        last = std::max(last, (q.isSatisfiedBy(*g.second.query) ? g.second.start : time_point::min()));
+    return last;
+}
+
+time_point
+Dht::Search::getLastGetTime() const
+{
+    time_point last = time_point::min();
+    for (const auto& g : callbacks)
+        last = std::max(last, g.second.start);
     return last;
 }
 
@@ -732,7 +805,7 @@ Dht::Search::isListening(time_point now) const
     for (const auto& n : nodes) {
         if (n.isBad())
             continue;
-        SearchNode::SyncStatus::const_iterator ls {};
+        SearchNode::NodeListenerStatus::const_iterator ls {};
         for (ls = n.listenStatus.begin(); ls != n.listenStatus.end() ; ++ls) {
             if (n.isListening(now, ls))
                 break;
