@@ -22,9 +22,9 @@
 #include "default_types.h"
 #include "securedht.h" // print certificate ID
 
-#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+#ifdef OPENDHT_JSONCPP
 #include "base64.h"
-#endif //OPENDHT_PROXY_SERVER
+#endif
 
 
 namespace dht {
@@ -172,43 +172,40 @@ Value::msgpack_unpack_body(const msgpack::object& o)
     }
 }
 
-#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
+#ifdef OPENDHT_JSONCPP
 Value::Value(Json::Value& json)
 {
-   try {
-       if (json.isMember("id"))
-           id = ValueType::Id(json["id"].asInt());
-   } catch (...) { }
-   if (json.isMember("cypher")) {
-       auto cypherStr = json["cypher"].asString();
-       cypherStr = base64_decode(cypherStr);
-       cypher = std::vector<unsigned char>(cypherStr.begin(), cypherStr.end());
-   }
-   if (json.isMember("sig")) {
-       auto sigStr = json["sig"].asString();
-       sigStr = base64_decode(sigStr);
-       signature = std::vector<unsigned char>(sigStr.begin(), sigStr.end());
-   }
-   if (json.isMember("seq"))
-       seq = json["seq"].asInt();
-   if (json.isMember("owner")) {
-       auto ownerStr = json["owner"].asString();
-       auto ownerBlob = std::vector<unsigned char>(ownerStr.begin(), ownerStr.end());
-       owner = std::make_shared<const crypto::PublicKey>(ownerBlob);
-   }
-   if (json.isMember("to")) {
-       auto toStr = json["to"].asString();
-       recipient = InfoHash(toStr);
-   }
-   if (json.isMember("type"))
-       type = json["type"].asInt();
-   if (json.isMember("data")){
-       auto dataStr = json["data"].asString();
-       dataStr = base64_decode(dataStr);
-       data = std::vector<unsigned char>(dataStr.begin(), dataStr.end());
-   }
-   if (json.isMember("utype"))
-       user_type = json["utype"].asString();
+    id = Value::Id(unpackId(json, "id"));
+    if (json.isMember("cypher")) {
+        auto cypherStr = json["cypher"].asString();
+        cypherStr = base64_decode(cypherStr);
+        cypher = std::vector<unsigned char>(cypherStr.begin(), cypherStr.end());
+    }
+    if (json.isMember("sig")) {
+        auto sigStr = json["sig"].asString();
+        sigStr = base64_decode(sigStr);
+        signature = std::vector<unsigned char>(sigStr.begin(), sigStr.end());
+    }
+    if (json.isMember("seq"))
+        seq = json["seq"].asInt();
+    if (json.isMember("owner")) {
+        auto ownerStr = json["owner"].asString();
+        auto ownerBlob = std::vector<unsigned char>(ownerStr.begin(), ownerStr.end());
+        owner = std::make_shared<const crypto::PublicKey>(ownerBlob);
+    }
+    if (json.isMember("to")) {
+        auto toStr = json["to"].asString();
+        recipient = InfoHash(toStr);
+    }
+    if (json.isMember("type"))
+        type = json["type"].asInt();
+    if (json.isMember("data")){
+        auto dataStr = json["data"].asString();
+        dataStr = base64_decode(dataStr);
+        data = std::vector<unsigned char>(dataStr.begin(), dataStr.end());
+    }
+    if (json.isMember("utype"))
+        user_type = json["utype"].asString();
 }
 
 Json::Value
@@ -225,7 +222,7 @@ Value::toJson() const
         if (has_owner) { // isSigned
             val["seq"] = seq;
             val["owner"] = owner->toString();
-            if (recipient != InfoHash())
+            if (recipient)
                 val["to"] = recipient.toString();
         }
         val["type"] = type;
@@ -235,7 +232,23 @@ Value::toJson() const
     }
     return val;
 }
-#endif //OPENDHT_PROXY_SERVER
+
+uint64_t
+unpackId(const Json::Value& json, const std::string& key) {
+    uint64_t ret = 0;
+    try {
+        if (json.isMember(key)) {
+            const auto& t = json[key];
+            if (t.isString()) {
+                ret = std::stoull(t.asString());
+            } else {
+                ret = t.asLargestUInt();
+            }
+        }
+    } catch (...) {}
+    return ret;
+}
+#endif
 
 bool
 FieldValue::operator==(const FieldValue& vfd) const
