@@ -72,12 +72,10 @@ void print_help() {
     std::cout << std::endl << "Operations with the proxy:" << std::endl
 #if OPENDHT_PUSH_NOTIFICATIONS
               << "  stt [server_address] <device_key> Start the proxy client." << std::endl
+              << "  rs  [token]                       Resubscribe to opendht." << std::endl
+              << "  rp  [token]                       Inject a push notification in Opendht." << std::endl
 #else
               << "  stt [server_address]              Start the proxy client." << std::endl
-#endif // OPENDHT_PUSH_NOTIFICATIONS
-#if OPENDHT_PUSH_NOTIFICATIONS
-              << "  rs  [token]                       Resubscribe to opendht." << std::endl
-              << "  rp  [push_notification]           Inject a push notification in Opendht." << std::endl
 #endif // OPENDHT_PUSH_NOTIFICATIONS
               << "  stp                               Stop the proxy client." << std::endl;
 #endif //OPENDHT_PROXY_CLIENT
@@ -89,14 +87,18 @@ void print_help() {
               << "  cl <key> <token>      Cancel listen for <token> and <key>." << std::endl
               << "  p <key> <str>         Put string value at <key>." << std::endl
               << "  pp <key> <str>        Put string value at <key> (persistent version)." << std::endl
+              << "  cpp <key> <id>        Cancel persistent put operation for <key> and value <id>." << std::endl
               << "  s <key> <str>         Put string value at <key>, signed with our generated private key." << std::endl
-              << "  e <key> <dest> <str>  Put string value at <key>, encrypted for <dest> with its public key (if found)." << std::endl;
+              << "  e <key> <dest> <str>  Put string value at <key>, encrypted for <dest> with its public key (if found)." << std::endl
+              << "  cc                    Trigger connectivity changed signal." << std::endl;
 
+#ifdef OPENDHT_INDEXATION
     std::cout << std::endl << "Indexation operations on the DHT:" << std::endl
               << "  il <name> <key> [exact match]   Lookup the index named <name> with the key <key>." << std::endl
               << "                                  Set [exact match] to 'false' for inexact match lookup." << std::endl
               << "  ii <name> <key> <value>         Inserts the value <value> under the key <key> in the index named <name>." << std::endl
               << std::endl;
+#endif
 }
 
 void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
@@ -113,7 +115,9 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
     using_history();
 #endif
 
+#ifdef OPENDHT_INDEXATION
     std::map<std::string, indexation::Pht> indexes;
+#endif
 
     while (true)
     {
@@ -137,6 +141,12 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
             std::cout << dht->getNodesStats(AF_INET).toString() << std::endl;
             std::cout << "IPv6 stats:" << std::endl;
             std::cout << dht->getNodesStats(AF_INET6).toString() << std::endl;
+#if OPENDHT_PROXY_SERVER
+            for (const auto& proxy : proxies) {
+                std::cout << "Stats for proxy on port " << proxy.first << std::endl;
+                std::cout << "  " << proxy.second->getStats().toString() << std::endl;
+            }
+#endif
             continue;
         } else if (op == "lr") {
             std::cout << "IPv4 routing table:" << std::endl;
@@ -170,11 +180,8 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
             iss >> idstr;
             try {
                 auto addr = splitPort(idstr);
-                if (not addr.first.empty() and addr.second.empty()){
-                    std::stringstream ss;
-                    ss << DHT_DEFAULT_PORT;
-                    addr.second = ss.str();
-                }
+                if (not addr.first.empty() and addr.second.empty())
+                    addr.second = std::to_string(DHT_DEFAULT_PORT);
                 dht->bootstrap(addr.first.c_str(), addr.second.c_str());
             } catch (const std::exception& e) {
                 std::cerr << e.what() << std::endl;
@@ -190,6 +197,9 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
                 log::disableLogging(*dht);
             dht->setLogFilter(filter);
             continue;
+        } else if (op == "cc") {
+            dht->connectivityChanged();
+            continue;
         }
 #if OPENDHT_PROXY_SERVER
         else if (op == "pst") {
@@ -228,14 +238,7 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
 #if OPENDHT_PUSH_NOTIFICATIONS
         else if (op == "rp") {
             iss >> value;
-            dht->pushNotificationReceived({{"token", value}});
-            continue;
-        } else if (op == "re") {
-            iss >> value;
-            try {
-                unsigned token = std::stoul(value);
-                dht->resubscribe(token);
-            } catch (...) { }
+            dht->pushNotificationReceived({{"to", "dhtnode"}, {"token", value}});
             continue;
         }
 #endif // OPENDHT_PUSH_NOTIFICATIONS
@@ -252,7 +255,9 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
         }
         dht::InfoHash id;
 
-        if (op == "il" or op == "ii") {
+        if (false) {}
+#ifdef OPENDHT_INDEXATION
+        else if (op == "il" or op == "ii") {
             // Pht syntax
             iss >> index >> keystr;
             auto new_index = std::find_if(indexes.begin(), indexes.end(),
@@ -260,7 +265,7 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
                         return i.first == index;
                     }) == indexes.end();
             if (not index.size()) {
-                std::cout << "You must enter the index name." << std::endl;
+                std::cerr << "You must enter the index name." << std::endl;
                 continue;
             } else if (new_index) {
                 using namespace dht::indexation;
@@ -273,11 +278,8 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
                     indexes.emplace(index, Pht {index, std::move(ks), dht});
                 } catch (std::invalid_argument& e) { std::cout << e.what() << std::endl; }
             }
-        } else if (op == "cl") {
-            std::string hash, rem;
-            iss >> hash >> rem;
-            dht->cancelListen(dht::InfoHash(hash), std::stoul(rem));
         }
+#endif
         else {
             // Dht syntax
             iss >> idstr;
@@ -325,14 +327,27 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
         else if (op == "l") {
             std::string rem;
             std::getline(iss, rem);
-            auto token = dht->listen(id, [](std::shared_ptr<Value> value) {
-                std::cout << "Listen: found value:" << std::endl;
-                std::cout << "\t" << *value << std::endl;
+            auto token = dht->listen(id, [](const std::vector<std::shared_ptr<Value>>& values, bool expired) {
+                std::cout << "Listen: found " << values.size() << " values" << (expired ? " expired" : "") << std::endl;
+                for (const auto& value : values)
+                    std::cout << "\t" << *value << std::endl;
                 return true;
             }, {}, dht::Where {std::move(rem)});
             auto t = token.get();
             std::cout << "Listening, token: " << t << std::endl;
         }
+        if (op == "cl") {
+            std::string rem;
+            iss >> rem;
+            size_t token;
+            try {
+                token = std::stoul(rem);
+            } catch(...) {
+                std::cerr << "Syntax: cl [key] [token]" << std::endl;
+                continue;
+            }
+            dht->cancelListen(id, token);
+        }
         else if (op == "p") {
             std::string v;
             iss >> v;
@@ -402,6 +417,7 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
                 std::cout << "Announce: " << (ok ? "success" : "failure") << " (took " << print_dt(end-start) << "s)" << std::endl;
             });
         }
+#ifdef OPENDHT_INDEXATION
         else if (op == "il") {
             std::string exact_match;
             iss >> exact_match;
@@ -447,6 +463,7 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
             catch (std::invalid_argument& e) { std::cout << e.what() << std::endl; }
             catch (std::out_of_range& e) { }
         }
+#endif
     }
 
     std::cout << std::endl <<  "Stopping node..." << std::endl;
@@ -480,8 +497,9 @@ main(int argc, char **argv)
             crt = dht::crypto::generateIdentity("DHT Node", ca_tmp);
         }
 
-        dht::DhtRunner::Config config;
+        dht::DhtRunner::Config config {};
         config.dht_config.node_config.network = params.network;
+        config.dht_config.node_config.maintain_storage = false;
         config.dht_config.id = crt;
         config.threaded = true;
         config.proxy_server = params.proxyclient;
